///////////////////////////////////////////////////////////////////////////
////                             LCDD.C                                ////
////                 Driver for common LCD modules                     ////
////                                                                   ////
////  lcd_init()   Must be called before any other function.           ////
////                                                                   ////
////  lcd_putc(c)  Will display c on the next position of the LCD.     ////
////                     The following have special meaning:           ////
////                      \f  Clear display                            ////
////                      \n  Go to start of second line               ////
////                      \b  Move back one position                   ////
////                                                                   ////
////  lcd_gotoxy(x,y) Set write position on LCD (upper left is 1,1)    ////
////                                                                   ////
////  lcd_getc(x,y)   Returns character at position x,y on LCD         ////
////                                                                   ////
///////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,2003 Custom Computer Services           ////
//// This source code may only be used by licensed users of the CCS C  ////
//// compiler.  This source code may only be distributed to other      ////
//// licensed users of the CCS C compiler.  No other use, reproduction ////
//// or distribution is permitted without written permission.          ////
//// Derivative programs created using this software in object code    ////
//// form are not restricted in any way.                               ////
///////////////////////////////////////////////////////////////////////////
/*
	051213 LCDhCo JX^}CY
	       5̓VA[pɃJX^}CY T.Nakao
*/

//============== C1 ================
#include "..\Common\reg877a.h"

// I/O|[gATC
#define LCD_E  PD.B1		// RD1
#define LCD_RS PD.B3		// RD3
#define LCD_RW PD.B2		// RD2

#define LCD_DATA PORTD;

struct {
	unsigned char dummy:4;		// b0-b3
	unsigned char Data:4;		// b4-b7
} _LCD;
#byte _LCD = 8					// PORTB

#define LCD_DATA_READ  0b11110000	// PORTD D7-D4:Input
#define LCD_DATA_WRITE 0b00001111	// PORTD D7-D4:Output

//============== C1 ================


// As defined in the following structure the pin connection is as follows:
//     D0  enable
//     D1  rs
//     D2  rw
//     D4  D4
//     D5  D5
//     D6  D6
//     D7  D7
//
//   LCD pins D0-D3 are not used and PIC D3 is not used.

// Un-comment the following define to use port B
// #define use_portb_lcd TRUE


//============== C2 ================
//struct lcd_pin_map {                 // This structure is overlayed
//           BOOLEAN enable;           // on to an I/O port to gain
//           BOOLEAN rs;               // access to the LCD pins.
//           BOOLEAN rw;               // The bits are allocated from
//           BOOLEAN unused;           // low order up.  ENABLE will
//           int     data : 4;         // be pin B0.
//        } lcd;
//============== C2 ================


#if defined(__PCH__)
#if defined use_portb_lcd
   #byte lcd = 0xF81                   // This puts the entire structure
#else
   #byte lcd = 0xF83                   // This puts the entire structure
#endif
#else
#if defined use_portb_lcd
   #byte lcd = 6                  // on to port B (at address 6)
#else
   #byte lcd = 8                 // on to port D (at address 8)
#endif
#endif

#if defined use_portb_lcd
   #define set_tris_lcd(x) set_tris_b(x)
#else
   #define set_tris_lcd(x) set_tris_d(x)
#endif


#define lcd_type 2           // 0=5x7, 1=5x10, 2=2 lines
#define lcd_line_two 0x40    // LCD RAM address for the second line


BYTE const LCD_INIT_STRING[4] = {0x20 | (lcd_type << 2), 0xc, 1, 6};
                             // These bytes need to be sent to the LCD
                             // to start it up.


                             // The following are used for setting
                             // the I/O port direction register.


//============== C3 ================
//struct lcd_pin_map const LCD_WRITE = {0,0,0,0,0}; // For write mode all pins are out
//struct lcd_pin_map const LCD_READ = {0,0,0,0,15}; // For read mode data pins are in
//============== C3 ================



BYTE lcd_read_byte() {
      BYTE low,high;
//============== C4 ================
//      set_tris_lcd(LCD_READ);
	TRISD |= LCD_DATA_READ;
//============== C4 ================
      LCD_RW = 1;
      delay_cycles(1);
      LCD_E = 1;
      delay_cycles(1);
      high = _LCD.Data;
      LCD_E = 0;
      delay_cycles(1);
      LCD_E = 1;
      delay_us(1);
      low = _LCD.Data;
      LCD_E = 0;
//============== C5 ================
//      set_tris_lcd(LCD_WRITE);
	TRISD &= LCD_DATA_WRITE;
//============== C5 ================
      return( (high<<4) | low);
}


void lcd_send_nibble( BYTE n ) {
      _LCD.Data = n;
      delay_cycles(1);
      LCD_E = 1;
      delay_us(2);
      LCD_E = 0;
}


void lcd_send_byte( BYTE address, BYTE n ) {

      LCD_RS = 0;
      while ( bit_test(lcd_read_byte(),7) ) ;
      LCD_RS = address;
      delay_cycles(1);
      LCD_RW = 0;
      delay_cycles(1);
      LCD_E = 0;
      lcd_send_nibble(n >> 4);
      lcd_send_nibble(n & 0xf);
}


void lcd_init() {
    BYTE i;
//============== C6 ================
//   set_tris_lcd(LCD_WRITE);
	TRISD &= LCD_DATA_WRITE;
	TRISD &= 0b11110001;				// b3-b1 output
//============== C6 ================
    LCD_RS = 0;
    LCD_RW = 0;
    LCD_E = 0;
    delay_ms(15);
    for(i=1;i<=3;++i) {
       lcd_send_nibble(3);
       delay_ms(5);
    }
    lcd_send_nibble(2);
    for(i=0;i<=3;++i)
       lcd_send_byte(0,LCD_INIT_STRING[i]);
}


void lcd_gotoxy( BYTE x, BYTE y) {
   BYTE address;

   if(y!=1)
     address=lcd_line_two;
   else
     address=0;
   address+=x-1;
   lcd_send_byte(0,0x80|address);
}

void lcd_putc( char c) {
   switch (c) {
     case '\f'   : lcd_send_byte(0,1);
                   delay_ms(2);
                                           break;
     case '\n'   : lcd_gotoxy(1,2);        break;
     case '\b'   : lcd_send_byte(0,0x10);  break;
     default     : lcd_send_byte(1,c);     break;
   }
}

char lcd_getc( BYTE x, BYTE y) {
   char value;

    lcd_gotoxy(x,y);
    while ( bit_test(lcd_read_byte(),7) ); // wait until busy flag is low
    LCD_RS=1;
    value = lcd_read_byte();
    LCD_RS=0;
    return(value);
}
